@@ -6,7 +6,7 @@ from .matching import match_style
 from .utils import pathlike
 
 
-def del_by_path(obj, path, match_with="glob", recursive_match_all=True, **kwargs):
+def del_by_path(obj, path, **kwargs):
     """Delete a key-value in a nested object in root by item sequence.
     from https://stackoverflow.com/a/14692747/913080
 
@@ -15,7 +15,7 @@ def del_by_path(obj, path, match_with="glob", recursive_match_all=True, **kwargs
     >>> obj
     {'a': [{'c': 'd'}]}
     """
-    del getitem_by_path(obj, path[:-1], match_with, recursive_match_all, **kwargs)[path[-1]]
+    del getitem_by_path(obj, path[:-1], **kwargs)[path[-1]]
 
 
 def getitem_by_path_strict(obj, path):
@@ -76,54 +76,53 @@ def _simplify_double_splats(path):
     return path
 
 
+def _resolve_double_splat(obj, path, *args, **kwargs):
+    # '**' can match any a path fragment of any length, including 0.
+    double_splat_idx = path.index("**") + 1
+
+    path_start = path[: double_splat_idx - 1]
+    path_end = path[double_splat_idx:]
+
+    if path_start:
+        paths = list(
+            resolve_path(
+                obj,
+                path_start,
+                *args,
+                **kwargs,
+            )
+        )
+        for p in paths:
+            path_ends = list(
+                # Use strict because the `paths` were just resolved
+                paths_to_key(
+                    getitem_by_path_strict(obj, p),
+                    path_end,
+                    *args,
+                    **kwargs,
+                )
+            )
+            for end in path_ends:
+                yield p + end
+    else:  # path started with "**"
+        path_ends = list(
+            paths_to_key(
+                obj,
+                path_end,
+                *args,
+                **kwargs,
+            )
+        )
+        yield from path_ends
+
+
 def resolve_path(obj, path, *args, match_with="glob", recursive_match_all=True, **kwargs):
     """Yield all paths that match the given globbed path."""
     if recursive_match_all and "**" in path:
         path = _simplify_double_splats(path)
 
     if recursive_match_all and "**" in path:
-        # '**' can match any a path fragment of any length, including 0.
-        double_splat_idx = path.index("**") + 1
-
-        path_start = path[: double_splat_idx - 1]
-        path_end = path[double_splat_idx:]
-
-        if path_start:
-            paths = list(
-                resolve_path(
-                    obj,
-                    path_start,
-                    *args,
-                    match_with=match_with,
-                    recursive_match_all=recursive_match_all,
-                    **kwargs,
-                )
-            )
-            for p in paths:
-                path_ends = list(
-                    paths_to_key(
-                        getitem_by_path_strict(obj, p),
-                        path_end,
-                        *args,
-                        match_with=match_with,
-                        recursive_match_all=recursive_match_all,
-                        **kwargs,
-                    )
-                )
-                for end in path_ends:
-                    yield p + end
-        else:  # path started with "**"
-            path_ends = list(
-                paths_to_key(
-                    obj,
-                    path_end,
-                    *args,
-                    match_with=match_with,
-                    recursive_match_all=recursive_match_all,
-                    **kwargs,
-                )
-            )
-            yield from path_ends
+        yield from _resolve_double_splat(obj, path, *args, **kwargs)
     elif path:
         first_step = path[0]
         path_remainder = path[1:]
@@ -133,11 +132,11 @@ def resolve_path(obj, path, *args, match_with="glob", recursive_match_all=True,
         if path_remainder:
             for key in keys:
                 sub_obj = obj[key]
-                smk = matched_keys(sub_obj, path_remainder[0], match_with, **kwargs)
-                if smk:
-                    yv = (
-                        [key] + sk
-                        for sk in resolve_path(
+                sub_keys = matched_keys(sub_obj, path_remainder[0], match_with, **kwargs)
+                if sub_keys:
+                    yield_values = (
+                        [key] + sub_key
+                        for sub_key in resolve_path(
                             sub_obj,
                             path_remainder,
                             *args,
@@ -146,7 +145,7 @@ def resolve_path(obj, path, *args, match_with="glob", recursive_match_all=True,
                             **kwargs,
                         )
                     )
-                    yield from yv
+                    yield from yield_values
         else:
             yield from ([key] for key in keys)
 
@@ -187,7 +186,10 @@ def matched_keys(obj, pattern, *args, match_with="glob", **kwargs):
     return rv
 
 
-def getitem_by_path(obj, path, *args, match_with="glob", **kwargs):
+def getitem_by_path(obj, path, *args, match_with="glob", strict=False, **kwargs):
+    if strict:
+        return getitem_by_path_strict(obj, path)
+
     if not pathlike(path):  # e.g. str or int
         if not match_style(match_with).patterned(path, *args, **kwargs):
             return obj[path]
@@ -200,16 +202,18 @@ def getitem_by_path(obj, path, *args, match_with="glob", **kwargs):
     elif len(paths) == 1:
         return getitem_by_path_strict(obj, paths[0])
 
+    # len(paths) == 0
     # Check if any part of the path appears to use a pattern. If so, return an empty
-    # list, rather than a KeyError/IndexError because 0 matched results.
-
+    # list, rather than a KeyError/IndexError because since a pattern was given, we
+    # are expecting a list of search results, which can be empty, rather than a
+    # strict retrieval.
     if any(match_style(match_with).patterned(p, *args, **kwargs) for p in path):
-        return paths
+        return []
 
     return getitem_by_path_strict(obj, path)
 
 
-def set_by_path(obj, path, value, *args, match_with="glob", recursive_match_all=True, **kwargs):
+def set_by_path(obj, path, value, *args, **kwargs):
     """Set a value in a nested object in obj by iterable path.
 
     If the path doesn't fully exist, this will create it to set the value,
@@ -232,13 +236,10 @@ def set_by_path(obj, path, value, *args, match_with="glob", recursive_match_all=
 
     traversed = []
     for part in path:
-        # branch = get_by_path(obj, [part])
         branch = getitem_by_path(
             obj,
             traversed,
             *args,
-            match_with=match_with,
-            recursive_match_all=recursive_match_all,
             **kwargs,
         )
         traversed.append(part)
@@ -297,13 +298,13 @@ def _paths_to_simple_key(obj, key, *args, match_with, _current, **kwargs):
     try:
         for k, v in obj.items():
             if pathlike(v):
-                yield from paths_to_key(v, key, match_with=match_with, _current=_current + [k], **kwargs)
+                yield from paths_to_key(v, key, *args, match_with=match_with, _current=_current + [k], **kwargs)
             if match_func(k, key):
                 yield _current + [k]
     except AttributeError:  # no .items
         for idx, v in enumerate(obj):
             if pathlike(v):
-                yield from paths_to_key(v, key, match_with=match_with, _current=_current + [idx], **kwargs)
+                yield from paths_to_key(v, key, *args, match_with=match_with, _current=_current + [idx], **kwargs)
             elif match_func(idx, key):
                 yield _current + [idx]
 
@@ -598,11 +599,12 @@ class DeepCollection(metaclass=DynamicSubclasser):
         self,
         obj,
         *args,
-        return_deep=True,
-        match_with="glob",
-        recursive_match_all=True,
         match_args=None,
         match_kwargs=None,
+        match_with="glob",
+        recursive_match_all=True,
+        return_deep=True,
+        strict=False,
         **kwargs,
     ):
         # Set instance vars first in case anything else (like super().__init__) accesses
@@ -617,12 +619,13 @@ class DeepCollection(metaclass=DynamicSubclasser):
         else:
             self._obj = obj
 
-        self.original_type = type(self._obj)
-        self.return_deep = return_deep
-        self.match_with = match_with
-        self.recursive_match_all = recursive_match_all
         self.match_args = match_args or ()
         self.match_kwargs = match_kwargs or {}
+        self.match_with = match_with
+        self.original_type = type(self._obj)
+        self.recursive_match_all = recursive_match_all
+        self.return_deep = return_deep
+        self.strict = strict
 
         # This often sets the original value for `self` for mutable types.
         # I.e. it gives a new list its content.
@@ -713,6 +716,7 @@ class DeepCollection(metaclass=DynamicSubclasser):
             *self.match_args,
             match_with=self.match_with,
             recursive_match_all=self.recursive_match_all,
+            strict=self.strict,
             **self.match_kwargs,
         )
 
@@ -723,6 +727,7 @@ class DeepCollection(metaclass=DynamicSubclasser):
                 recursive_match_all=self.recursive_match_all,
                 match_args=self.match_args,
                 match_kwargs=self.match_kwargs,
+                strict=self.strict,
             )
         return rv
 
@@ -734,6 +739,7 @@ class DeepCollection(metaclass=DynamicSubclasser):
                 *self.match_args,
                 match_with=self.match_with,
                 recursive_match_all=self.recursive_match_all,
+                strict=self.strict,
                 **self.match_kwargs,
             )
         else:
@@ -749,6 +755,7 @@ class DeepCollection(metaclass=DynamicSubclasser):
                 *self.match_args,
                 match_with=self.match_with,
                 recursive_match_all=self.recursive_match_all,
+                strict=self.strict,
                 **self.match_kwargs,
             )
         else:
@@ -763,13 +770,25 @@ class DeepCollection(metaclass=DynamicSubclasser):
         return f"DeepCollection({super_repr})"
 
     # Common public methods
-    def get(self, path, default=None, *, match_args=None, match_with=None, recursive_match_all=None, match_kwargs=None):
+    def get(
+        self,
+        path,
+        default=None,
+        *,
+        match_args=None,
+        match_with=None,
+        recursive_match_all=None,
+        match_kwargs=None,
+        strict=None,
+    ):
         # These are one-offs and should not mutate self
         match_args = match_args or self.match_args
         match_with = match_with or self.match_with
         match_kwargs = match_kwargs or self.match_kwargs
         if recursive_match_all is None:
             recursive_match_all = self.recursive_match_all
+        if strict is None:
+            strict = self.strict
 
         try:
             rv = getitem_by_path(
@@ -778,6 +797,7 @@ class DeepCollection(metaclass=DynamicSubclasser):
                 *match_args,
                 match_with=match_with,
                 recursive_match_all=recursive_match_all,
+                strict=strict,
                 **match_kwargs,
             )
         except (KeyError, IndexError, TypeError):
@@ -791,6 +811,7 @@ class DeepCollection(metaclass=DynamicSubclasser):
                 recursive_match_all=self.recursive_match_all,
                 match_args=self.match_args,
                 match_kwargs=self.match_kwargs,
+                strict=strict,
             )
         return rv
 
@@ -799,7 +820,7 @@ class DeepCollection(metaclass=DynamicSubclasser):
         return super().items(*args, **kwargs)
 
     # Unique public methods
-    def paths_to_key(self, key, *args, match_with=None, recursive_match_all=None, **kwargs):
+    def paths_to_key(self, key, *args, match_with=None, recursive_match_all=None, strict=None, **kwargs):
         """
         >>> list(DeepCollection([{"x": {"y": "value", "z": {"y": "asdf"}}}]).paths_to_key("y"))
         [[0, 'x', 'y'], [0, 'x', 'z', 'y']]
@@ -809,12 +830,35 @@ class DeepCollection(metaclass=DynamicSubclasser):
         match_kwargs = kwargs or self.match_kwargs
         if recursive_match_all is None:
             recursive_match_all = self.recursive_match_all
+        if strict is None:
+            strict = self.strict
 
         yield from paths_to_key(
+            self,
+            key,
+            *match_args,
+            match_with=match_with,
+            recursive_match_all=recursive_match_all,
+            strict=strict,
+            **match_kwargs,
+        )
+
+    def paths_to_value(self, key, *args, match_with=None, recursive_match_all=None, **kwargs):
+        """
+        >>> list(DeepCollection([{"x": {"y": "value", "z": {"y": "asdf"}}}]).paths_to_value("asdf"))
+        [[0, 'x', 'z', 'y']]
+        """
+        match_with = match_with or self.match_with
+        match_args = args or self.match_args
+        match_kwargs = kwargs or self.match_kwargs
+        if recursive_match_all is None:
+            recursive_match_all = self.recursive_match_all
+
+        yield from paths_to_value(
             self, key, *match_args, match_with=match_with, recursive_match_all=recursive_match_all, **match_kwargs
         )
 
-    def values_for_key(self, key, *args, match_with=None, recursive_match_all=None, **kwargs):
+    def values_for_key(self, key, *args, match_with=None, recursive_match_all=None, strict=None, **kwargs):
         """
         >>> dc = DeepCollection([{"x": {"y": "v", "z": {"y": "v"}}, "y": {1: 2}}], return_deep=False)
         >>> list(dc.values_for_key("y"))
@@ -825,12 +869,20 @@ class DeepCollection(metaclass=DynamicSubclasser):
         match_kwargs = kwargs or self.match_kwargs
         if recursive_match_all is None:
             recursive_match_all = self.recursive_match_all
+        if strict is None:
+            strict = self.strict
 
         yield from values_for_key(
-            self, key, *match_args, match_with=match_with, recursive_match_all=recursive_match_all, **match_kwargs
+            self,
+            key,
+            *match_args,
+            match_with=match_with,
+            recursive_match_all=recursive_match_all,
+            strict=strict,
+            **match_kwargs,
         )
 
-    def deduped_values_for_key(self, key, *args, match_with=None, recursive_match_all=None, **kwargs):
+    def deduped_values_for_key(self, key, *args, match_with=None, recursive_match_all=None, strict=None, **kwargs):
         """
         >>> dc = DeepCollection([{"x": {"y": "v", "z": {"y": "v"}}, "y": {1: 2}}], return_deep=False)
         >>> 'v' in dc.deduped_values_for_key("y")  # order not gaurunteed
@@ -843,6 +895,8 @@ class DeepCollection(metaclass=DynamicSubclasser):
         match_kwargs = kwargs or self.match_kwargs
         if recursive_match_all is None:
             recursive_match_all = self.recursive_match_all
+        if strict is None:
+            strict = self.strict
 
         return deduped_items(
             list(
@@ -852,6 +906,7 @@ class DeepCollection(metaclass=DynamicSubclasser):
                     *match_args,
                     match_with=match_with,
                     recursive_match_all=recursive_match_all,
+                    strict=strict,
                     **match_kwargs,
                 )
             )
